import json
import pkgutil

import pandas as pd

from vega_datasets._compat import urlopen, BytesIO, lru_cache, bytes_decode


class Dataset(object):
    """Class to extract and orgainize information about a dataset

    Parameters
    ----------
    name : string
        The name of the dataset. This should be one of the options available
        in Dataset.list_datasets()

    Attributes
    ----------
    filename : string
        The filename in which the dataset is stored
    url : string
        The full URL of the dataset at http://vega.github.io
    format : string
        The format of the dataset: usually one of {'csv', 'tsv', 'json'}
    pkg_filename : string
        The path to the local dataset within the vega_datasets package
    is_local : bool
        True if the dataset is available locally in the package
    """
    base_url = 'https://vega.github.io/vega-datasets/data/'

    def __init__(self, name):
        info = self._infodict(name)
        self.filename = info['filename']
        self.url = self.base_url + info['filename']
        self.format = info['format']
        self.pkg_filename = 'data/' + self.filename

    @classmethod
    def list_datasets(cls):
        """Return a list of names of available datasets"""
        return sorted(cls._datasets_json().keys())

    @classmethod
    @lru_cache()
    def _datasets_json(cls):
        """load the datasets.json file"""
        datasets = pkgutil.get_data('vega_datasets', 'datasets.json')
        return json.loads(bytes_decode(datasets))

    @classmethod
    @lru_cache()
    def _infodict(cls, name):
        """load the info dictionary for the given name"""
        info = cls._datasets_json().get(name, None)
        if info is None:
            raise ValueError('No such dataset {0} exists, '
                             'use list_datasets() to get a list '
                             'of available datasets.'.format(name))
        return info

    @property
    def is_local(self):
        try:
            pkgutil.get_data('vega_datasets', self.pkg_filename)
        except FileNotFoundError:
            return False
        else:
            return True

    def load(self, return_raw=False, use_local=True):
        """Load the dataset from remote URL or local file

        Parameters
        ----------
        return_raw : boolean
            If True, then return the raw string or bytes.
            If False (default), then return a pandas dataframe.
        use_local : boolean
            If True (default), then attempt to load the dataset locally. If
            False or if the dataset is not available locally, then load the
            data from an external URL.
        """
        if use_local and self.is_local:
            data = BytesIO(pkgutil.get_data('vega_datasets', self.pkg_filename))
        else:
            data = urlopen(self.url)

        if return_raw:
            return data.read()
        elif self.format == 'json':
            return pd.read_json(data)
        elif self.format == 'csv':
            return pd.read_csv(data)
        elif self.format == 'tsv':
            return pd.read_csv(data, sep='\t')
        else:
            raise ValueError("Unrecognized file format: {0}. "
                             "Valid options are ['json', 'csv', 'tsv']."
                             "".format(self.format))


def list_datasets():
    """Return a list of the available dataset names."""
    return Dataset.list_datasets()


def data(name, return_raw=False, use_local=True):
    """Load a dataset from a local file or remote URL.

    Parameters
    ----------
    name : string
        The name of the dataset. This should be one of the options available
        in list_datasets()
    return_raw : boolean
        If True, then return the raw string or bytes.
        If False (default), then return a pandas dataframe.
    use_local : boolean
        If True (default), then attempt to load the dataset locally. If
        False or if the dataset is not available locally, then load the
        data from an external URL.
    """
    return Dataset(name).load(return_raw=return_raw, use_local=use_local)



import pandas as pd

import pytest

from vega_datasets import data, Dataset
from vega_datasets.utils import connection_ok


def test_local_iris():
    assert Dataset('iris').is_local

    iris = data('iris', use_local=True)
    assert type(iris) is pd.DataFrame
    assert tuple(iris.columns) == ('petalLength', 'petalWidth', 'sepalLength',
                                   'sepalWidth', 'species')

    iris = data('iris', use_local=True, return_raw=True)
    assert type(iris) is bytes


@pytest.mark.skipif(not connection_ok(), reason="No internet connection")
def test_download_iris():
    iris = data('iris', use_local=False)
    assert type(iris) is pd.DataFrame
    assert tuple(iris.columns) == ('petalLength', 'petalWidth', 'sepalLength',
                                   'sepalWidth', 'species')

    iris = data('iris', use_local=False, return_raw=True)
    assert type(iris) is bytes



import io
import os
import re

from setuptools import setup


def read(path, encoding='utf-8'):
    path = os.path.join(os.path.dirname(__file__), path)
    with io.open(path, encoding=encoding) as fp:
        return fp.read()


def version(path):
    """Obtain the packge version from a python file e.g. pkg/__init__.py
    See <https://packaging.python.org/en/latest/single_source_version.html>.
    """
    version_file = read(path)
    version_match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


DESCRIPTION = "A Python package for offline access to Vega datasets"
LONG_DESCRIPTION = """
A Python package for offline access to Vega datasets available at
https://github.com/vega/vega-datasets.

This package has several goals:

- Provide straightforward access to the datasets in vega-datasets
- return the results in the form of a Pandas dataframe.
- wherever dataset size and/or license constraints make it possible, bundle the dataset with the package so that datasets can be loaded in the absence of a web connection.

Development at http://github.com/jakevdp/vega_datasets
"""
NAME = "vega_datasets"
AUTHOR = "Jake VanderPlas"
AUTHOR_EMAIL = "jakevdp@gmail.com"
MAINTAINER = "Jake VanderPlas"
MAINTAINER_EMAIL = "jakevdp@gmail.com"
URL = 'http://github.com/jakevdp/vega_datasets'
DOWNLOAD_URL = 'http://github.com/jakevdp/vega_datasets'
LICENSE = 'MIT'

VERSION = version('vega_datasets/__init__.py')

setup(name=NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      url=URL,
      download_url=DOWNLOAD_URL,
      license=LICENSE,
      install_requires=["pandas"],
      tests_require=["pytest"],
      packages=['vega_datasets',
                'vega_datasets.tests',
            ],
      package_data={'vega_datasets': ['*.json',
                                      os.path.join('data', '*.json'),
                                      os.path.join('data', '*.csv'),
                                      os.path.join('data', '*.tsv')]},
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'],
     )
